;
; The following symbols are defined on the command line that invokes the assembler:
;
; BIGFLASH       ---  0 -> CANdo 68376 has 2 each 128K X 8 flash memory devices
;                     1 -> CANdo 68376 has 2 each 512K X 8 flash memory devices
;
; BIGSRAM        ---  0 -> CANdo 68376 has 2 each 128K X 8 sram memory devices
;                     1 -> CANdo 68376 has 2 each 512K X 8 sram memory devices
;
; MODULEBASE     ---  0xfff000 When the MM bit in SIMCR is = 1 (this is the reset condition)
;                     0x7ff000 When the MM bit in SIMCR is = 0 
;
	INCLUDE	<68376.inc>			; Defines MC68376 registers. Requires symbol MODULEBASE
	INCLUDE <CandoBug.inc>		; Specific equates for the CANdo 68376. Requires symbols:
	                            ; BIGFLASH, BIGSRAM
;
; All of the code that follows is position independent.
;
; List of programs and entry points:
;
; CAN bus demonstration program                CanBusDemo
; Flash memory erase program                   FlashErase
; Flash memory write program                   FlashWrite
;
; To execute these programs using CPU32BUG you will need the absolute
; memory address for the entry point. By inspecting the absolute listing 
; file you can find the addresses for the above listed entry points. Add this 
; entry point address to the base address of the flash memory. For the CANd0 68376
; module this would be:
;
; The symbol FLASHBASE is defined in CandoBug.inc.
; For both the big flash and small flash version of the CANdo 68376
; FLASHBASE = 0x100000
;
; CanBusDemo:  0xE300 + 0x100000 = 0x10E300
; FlashErase:  0xE558 + 0x100000 = 0x10E558
; FlashWrite:  0xE6D4 + 0x100000 = 0x10E6D4
;
; CAUTION! If the code is changed the entry points will change. So, be sure
; and inspect the absolute listing file if changes are made.
;
; To run the can bus demo program enter: G 10E300 at the CPU32BUG prompt.
;


	SECTION	EXTENSIONS
;
;
;
; *******************************************************************************************************************
; *******************************************************************************************************************
; *******************************************************************************************************************
;
;
;
; The folowing program configures the MC68376 TOUCAN module. If 2 or more CANdo 68376 modules
; are connected by the CAN bus, and each module is executing this program. Then
; characters typed on the terminal connected to any module is transmitted over the
; CAN bus and displayed by the other modules. To exit the demonstration program
; and return control to CPU32BUG type control C.
;
; Spectrum Engineering example code for demonstrating the TouCan module
;
CanBusDemo:

	jsr	(toucan_init,pc)		; setup the toucan module


	pea	message1(pc)
	trap	#15					; Use CPU32BUG system call to display message
	dc.w	0x24

;
;
;
try_can:
	jsr		(can_get,pc)		; test for a received CAN bus message
	cmpi.l	#-1,d0
	beq		try_sci				; if no message then test the sci
	cmpi.l	#0x0d,d0			; is the message a <CR> ?
	bne		notcr
	pea		crlfmsg(pc)			; got <CR> so send <CR><LF> out SCI
	trap	#15
	DC.W	0x24
	bra.l	try_sci
notcr:
	jsr		(sci_put,pc)		; send the character in d0 out the SCI port


try_sci:
	jsr		(sci_get,pc)		; test for SCI received character
	cmpi.l	#-1,d0
	beq		try_can				; if no received character then test the CAN bus
	cmpi.l	#('C'-0x40),d0
	bne		notctrlc
	move.w	#0x5980,canCANMCR	; HALT the TouCan module
	pea	message2(pc)			; Log offmessage
	trap	#15
	dc.w	0x24
	trap	#15					; CPU32BUG system call. return to CPU332BUG
	dc.w	0x63
notctrlc:
	jsr		(can_put,pc)		; transmit a CAN bus message with this character
	bra.l	try_can
;
;
;
;
can_get:
;
	move.w	canBUF1+0,d0
	andi.l	#0xf0,d0
	cmpi.l	#0x20,d0
	beq		can_get1
	move.w	canTIMER,d0
	move.l	#-1,d0
	rts
can_get1:
	move.w	canBUF1+6,d0
	move.w	#0x0040,canBUF1+0
	asr.l	#8,d0
	andi.l	#0xff,d0
	move.w	canTIMER,d1
	rts
;
;
;
;
can_put:
;
	asl.l	#8,d0
	move.w	d0,canBUF0+6
	move.w	#0x00c1,canBUF0+0
can_put1:
	move.w	canBUF0+0,d0
	andi.l	#0xf0,d0
	cmpi.l	#0x80,d0
	bne		can_put1
	rts
;
;
;
;
sci_get:
;
	move.w	qsmSCSR,d0
	btst	#6,d0
	beq		sci_get1
	move.w	qsmSCDR,d0
	andi.l	#0xff,d0
	rts
sci_get1:
	move.l	#-1,d0
	rts
;
;
;
;
sci_put:
;
	move.w	qsmSCSR,d1
	btst	#8,d1
	beq		sci_put
	move.w	d0,qsmSCDR
	rts


toucan_init:
;
;
;	 Assume SYSTEM_CLOCK = 16 MHz. We want the CAN bus to run at 250 KHz. 
;	 Set PRESDIV = 3, then time quanta clock = 4 MHz -> 1 TimeQuanta = 250ns
;
;	 A nominal bit time for CAN has 4 subparts:
;	 Sync segment
;	 Prop segment
;	 Phase segment 1
;	 Phase segment 2
;
;	 By CAN bus specification Sync segment =           1 Time Quantum
;	 Prop segment is programmed to 6, yields           7 Time Quantum
;	 Phase segment 1 is programmed to 3, yields        4 Time Quantum
;	 Phase segment 2 is programmed to 3, yields        4 TIme Quantum
;	 TOTAL                                            16 Time Quantum
;
;	 So, with 1 Time Quantum = 250ns, 16 Time Quantum = 4us for a nominal bit rate = 250 KHz
;
;
	move.b	#0x03,canPRESDIV	; OUT8( canPRESDIV, 3 )

; Let CANCTRL2 = 01 011 011 -> RJW = 1, PSEG1 = 3, PSEG2 = 3
; Phase buffer segment 1 = (PSEG1 + 1) * TimeQuanta
; = 4 * 250ns = 1us
; Phase buffer segment 2 = (PSEG2 + 1) * TimeQuanta
; = 4 * 250ns = 1us
; InSync segment defined by CANspec = 1 * TimeQuanta = 250ns
;
	move.b	#0x5b,canCANCTRL2	; OUT8( canCANCTRL2, 0x5b)

; Let CANCTRL1 = 0 0 0 0 0 110 -> SAMP = 0, LOOP = 0, TSYNC = 0, LBUF = 0, RSVD = 0, PROPSEG = 6

	move.b	#0x06,canCANCTRL1	; OUT8( canCANCTRL1, 0x06)

; CAN bit timing is complete.
;
; Buffer 0 transmits characters.

	move.w	#0x0000,canBUF0+0	; OUT16( canBUF0+0,  0x0000 )

; 29 bit id = 10100111011,110,010001000100001
; SRR bit = 1
; IDE bit = 1, means use 29 bit id
; RTR bit = 0
; 10100111011 1 1 110 010001000100001 0   =   0xa77e, 0x4442

	move.w	#0xa77e,canBUF0+2	; OUT16( canBUF0+2,  0xa77e )
	move.w	#0x4442,canBUF0+4	; OUT16( canBUF0+4,  0x4442 )


; Buffer 1 receives characters.

	move.w	#0x0000,canBUF1+0	; OUT16( canBUF1+0,  0x0000 )

; 29 bit id = 10100111011,110,010001000100001
; SRR bit = 1
; IDE bit = 1, means use 29 bit id
; RTR bit = 0
; 10100111011 1 1 110 010001000100001 0   =   0xa77e, 0x4442

	move.w	#0xa77e,canBUF1+2	; OUT16( canBUF1+2,  0xa77e )
	move.w	#0x4442,canBUF1+4	; OUT16( canBUF1+4,  0x4442 )

; Buffers 2 through 15 are not used

	move.w	#0x0000,canBUF2+0
	move.w	#0x0000,canBUF3+0
	move.w	#0x0000,canBUF4+0
	move.w	#0x0000,canBUF5+0
	move.w	#0x0000,canBUF6+0
	move.w	#0x0000,canBUF7+0
	move.w	#0x0000,canBUF8+0
	move.w	#0x0000,canBUF9+0
	move.w	#0x0000,canBUF10+0
	move.w	#0x0000,canBUF11+0
	move.w	#0x0000,canBUF12+0
	move.w	#0x0000,canBUF13+0
	move.w	#0x0000,canBUF14+0
	move.w	#0x0000,canBUF15+0

	move.w	#0x0040,canBUF1+0	; OUT16( canBUF1+0,  0x0040 ), Enable buffer 1 ready to receive
	move.w	#0x4980,canCANMCR	; OUT16( canCANMCR, 0x4980 ) ok, enable the toucan to run !! 
	rts


message2:	DC.B	m3b-m3a
m3a:
			DC.B	0x0a,0x0d,0x0d,0x0d,'Exit CAN bus example, return to CPU332BUG',0x0a,0x0d
			DC.B	'CAN TALK V1.0',0x0a,0x0d,0x0d
m3b:
;
message1:	DC.B	m1b-m1a
m1a:
			DC.B	0x0a,0x0d,0x0d,0x0d,'Spectrum Engineering, CAN bus example',0x0a,0x0d
			DC.B	'CAN TALK V1.0',0x0a,0x0d,0x0d
m1b:
;
crlfmsg:	DC.B	m2b-m2a
m2a:
			DC.B	0x0a,0x0d
m2b:
	.align	4
;
;
;
; *******************************************************************************************************************
; *******************************************************************************************************************
; *******************************************************************************************************************
;
;
;
; This program will erase a block of flash memory. Block 0 of flash has CPU32BUG along with
; this and other programs.
;
FlashErase:
;
	movea.l	#TPURAMBASE,a1		; TPU base address in a1
	movea.l	#0x0,a3				; use a3 for index register
;
.copy_loop:
	move.l	(.start_block,PC,a3.L*4),(a1)+
	addq.l	#1,a3
	cmpa.l	#.end_block-.start_block,a3
	bne		.copy_loop
;

	pea	.msg1(pc)
	trap	#15					; Use CPU32BUG system call to display message
	dc.w	0x24

.askforblock:	
	pea	.msg2(pc)
	trap	#15					; Use CPU32BUG system call to display message
	dc.w	0x23

	pea	userinput
	trap	#15
	dc.w	0x04

	cmpi.b	#0,userinput+0		; test for no input
	bne		.nonzero

	trap	#15					; CPU32BUG system call. return to CPU332BUG
	dc.w	0x63

.nonzero:

	move.b	userinput+1,d0
	andi.l	#0xf,d0
	.IF BIGFLASH
	cmpi.b	#0,d0
	.ELSE
	cmpi.b	#1,d0
	.ENDIF
	bgt	.ok0

	pea	.msg3(pc)				; flag error not allowed to erase block 0
	trap	#15
	dc.w	0x24
	bra		.askforblock

.ok0:
	cmpi.b	#7,d0
	ble		.ok1

	pea	.msg4(pc)				; flag error block greater than 7
	trap	#15
	dc.w	0x24
	bra		.askforblock



.ok1:
;
	jsr		TPURAMBASE
	bra		.askforblock



.msg1:	DC.B	.msg1b-.msg1a
.msg1a:
			DC.B	0x0a,0x0d,0x0d,0x0d,'Erase Flash memory blocks',0x0a,0x0d
.msg1b:

.msg2:	DC.B	.msg2b-.msg2a
.msg2a:
			DC.B	0x0a,0x0d,'Enter block number to erase:'
.msg2b:

.msg3:	DC.B	.msg3b-.msg3a
.msg3a:
			DC.B	0x0a,0x0d,'ERROR, block 0 has CPU32BUG (4mbit flash)',0x0a,0x0d
			DC.B	'ERROR, block 0 & 1 has CPU32BUG (1mbit flash)'
.msg3b:

.msg4:	DC.B	.msg4b-.msg4a
.msg4a:
			DC.B	0x0a,0x0d,'ERROR, block number too large'
.msg4b:
;
; This code will erase a selected Flash memory block. Enter with the block
; number in d0. Since the Flash memory is busy during an erase operation
; this code must be executed from SRAM.
;
; The following section of position independent code is stored in FLASH memory
; then copied to SRAM, then executed.

	.align	4
.start_block
	.IF BIGFLASH
	move.l	#17,d1
	.ELSE
	move.l	#15,d1
	.ENDIF


	lsl.l	d1,d0
	addi.l	#FLASHBASE,d0
	move.l	d0,a0
	move.w	sr,d1
	ori.w	#0x0700,sr
	.IF BIGFLASH
	move.w	#0xaaaa,(0xaaa+FLASHBASE)			; write #1
	move.w	#0x5555,(0x554+FLASHBASE)			; write #2
	move.w	#0x8080,(0xaaa+FLASHBASE)			; write #3
	move.w	#0xaaaa,(0xaaa+FLASHBASE)			; write #4
	move.w	#0x5555,(0x554+FLASHBASE)			; write #5
	move.w	#0x3030,(a0)						; write #6
	.ELSE
	move.w	#0xaaaa,(0xaaaa+FLASHBASE)			; write #1
	move.w	#0x5555,(0x5554+FLASHBASE)			; write #2
	move.w	#0x8080,(0xaaaa+FLASHBASE)			; write #3
	move.w	#0xaaaa,(0xaaaa+FLASHBASE)			; write #4
	move.w	#0x5555,(0x5554+FLASHBASE)			; write #5
	move.w	#0x3030,(a0)						; write #6
	.ENDIF
.wait1:
	cmpi.w	#0xffff,(a0)
	bne		.wait1
	move.w	d1,sr
	rts
.end_block
;
;
;
; *******************************************************************************************************************
; *******************************************************************************************************************
; *******************************************************************************************************************
;
;
;
; This program will program Flash memory. The user must initialize the following
; memory locations prior to calling this program.
;
;
; 1. The bytes to be programmed must first be loaded into SRAM using the CPU32BUG LOAD
;    command.
; 2. Initialize a0 to point to the first WORD in SRAM that is to be stored in Flash.
; 3. Initialize a1 to point to the last  WORD in SRAM that is to be stored in Flash.
; 4. Initialize a2 to point to the first WORD in Flash that is to be programmed.
;

FlashWrite:
;
	move.l	a0,-(a7)
	move.l	a1,-(a7)
	move.l	a2,-(a7)			; push a0,a1,a2 onto stack

	movea.l	#TPURAMBASE,a1		; TPU base address in a1
	movea.l	#0x0,a3				; use a3 for index register
;
.copy_loop:
	move.l	(.start_block,PC,a3.L*4),(a1)+
	addq.l	#1,a3
	cmpa.l	#.end_block-.start_block,a3
	bne		.copy_loop

	pea	.msg1(pc)
	trap	#15					; Use CPU32BUG system call to display message
	dc.w	0x24

	pea	.msg2(pc)
	trap	#15					; Use CPU32BUG system call to display message
	dc.w	0x23

	pea	userinput
	trap	#15
	dc.w	0x04

	cmpi.b	#0,userinput+0		; test for no input
	bne		.nonzero

	trap	#15					; CPU32BUG system call. return to CPU332BUG
	dc.w	0x63

.nonzero:
; Ok, the registers must be set up, so we will program the flash
;

	move.l	(a7)+,a2
	move.l	(a7)+,a1
	move.l	(a7)+,a0
	jsr		TPURAMBASE
	trap	#15					; CPU32BUG system call. return to CPU332BUG
	dc.w	0x63







.msg1:	DC.B	.msg1b-.msg1a
.msg1a:
			DC.B	0x0a,0x0d,0x0d,0x0d,'Program Flash memory from SRAM buffer',0x0a,0x0d
			DC.B	'Initialize a0 to first WORD address in SRAM buffer',0x0a,0x0d
			DC.B	'Initialize a1 to last WORD address in SRAM buffer',0x0a,0x0d
			DC.B	'Initialize a2 to first WORD address in Flash to program',0x0d,0x0a
.msg1b:

.msg2:	DC.B	.msg2b-.msg2a
.msg2a:
			DC.B	0x0a,0x0d,'To exit press RETURN, to program press 1 then RETURN'
.msg2b:


	.align	4
.start_block
.domore:
	move.w	sr,d1
	ori.w	#0x0700,sr
	.IF	BIGFLASH
	move.w	#0xaaaa,(0xaaa+FLASHBASE)			; write #1
	move.w	#0x5555,(0x554+FLASHBASE)			; write #2
	move.w	#0xa0a0,(0xaaa+FLASHBASE)			; write #3
	move.w	(a0),(a2)							; read word from sram, write to flash
	.ELSE
	move.w	#0xaaaa,(0xaaaa+FLASHBASE)			; write #1
	move.w	#0x5555,(0x5554+FLASHBASE)			; write #2
	move.w	#0xa0a0,(0xaaaa+FLASHBASE)			; write #3
	move.w	(a0),(a2)							; read word from sram, write to flash
	.ENDIF
.wait1:
	move.w	(a0),d0
	cmp.w	(a2),d0
	bne		.wait1
	cmpa.l	a0,a1
	beq		.done
	adda.l	#2,a0
	adda.l	#2,a2
	bra		.domore
.done:
	move.w	d1,sr
	rts
	.align	4
.end_block

	.org	0x3002				; start of user ram area

userinput:	DS.B	256			; user input string storage
