; The following symbols are defined on the command line that invokes the assembler:
;
; BIGFLASH       ---  0 -> CANdo 68376 has 2 each 128K X 8 flash memory devices
;                     1 -> CANdo 68376 has 2 each 512K X 8 flash memory devices
;
; BIGSRAM        ---  0 -> CANdo 68376 has 2 each 128K X 8 sram memory devices
;                     1 -> CANdo 68376 has 2 each 512K X 8 sram memory devices
;
; modulebase     ---  0xfff000 When the MM bit in SIMCR is = 1 (this is the reset condition)
;                     0x7ff000 When the MM bit in SIMCR is = 0 
;
; CHECKSUMWORD   --- Refer to the CPU32BUG manual to understand the checksum
;
	INCLUDE	<68376.inc>			; Defines MC68376 registers. Requires symbol MODULEBASE
	INCLUDE <CandoBug.inc>		; Specific equates for the CANdo 68376. Requires symbols:
	                            ; BIGFLASH, BIGSRAM
;
;
; This file defines the user customization area (parameter area) for the CPU32BUG
; monitor developed by Motorola. You should have the following documents in order to
; understand how the parameter area is used:
;
; cpu32bug.pdf, available from the Motorola web site or from the Spectrum Engineering
; web site: www.spectrumengineering.com. Appendix C of this manual explains the parameter
; area in detail.
;
; buglist.txt, this file is ARC'd with the various CPU32BUG files on the Motorola web site,
; it is also available from the Spectrum Engineering web site. This file lists know bugs
; in CPU32BUG.
;
; CPU32BUG as developed by Motorola consists of 3 sections:
; Parameter area in EPROM:     0xe0000 ... 0xe01ff
; CPU32BUG code area in EPROM: 0xe0200 ... 0xee1cf
; CPU32BUG RAM:                0x00000 ... 0x03000
;
; The CPU32BUG code is position independent, so can be placed anywhere within the 68376 
; address space.
;
; Memory map AFTER reset initialization:
;
;------------------------------------   0x00200E00
;                                   -
;                                   -
;   3.5K byte TPU Emulation ram     -
;                                   -   
;                                   -
;------------------------------------   0x00200000
;                                   -
;                                   -
;         1024K bytes FLASH         -
;                                   -   0x0010E1CF   ; CPU32BUG code
;                                   -
;------------------------------------   0x00100000
;
;
;------------------------------------   0x00040000
;                                   -
;                                   -
;         256K bytes SRAM           -
;                                   -   0x00003000   ; CPU32BUG & target vector table
;                                   -
;------------------------------------   0x00000000
;
; SRAM space 0x3000 to 0x40000 is available for user programs
; FLASH space 0x10E1D0 to 0x200000 is available for user programs


	SECTION	PARAMETERS


BEGIN_CHECKSUM_COUNT:

	DC.L	0x00002ffc				; PWR_SSP power on/reset stack pointer. The stack
; pointer is initialized with this 32 bit value after reset.


	DC.L	POWER_ON_RESET			; PWR_PC  power/on reset program counter.The program
; counter is initialized with this 32 bit value after reset.


	DC.L	(END_CHECKSUM_COUNT-BEGIN_CHECKSUM_COUNT)
; CODESIZE size of 332Bug ROM in bytes, must be even number

	DC.B	0x20					; SRECMAX max. number of data bytes for S-record

	DC.B	0xff					; CHECKALT

	DC.W	CHECKSUMWORD			; CHECKSUM

; Original CPU32Bug chip select table. None of these values are used. Chip select
; register initialization is accomplished in the POWER_ON_RESET routine.

	DC.W	0x0000					; CSBAR0
	DC.W	0x0000					; CSOR0
	DC.W	0x0000					; CSBAR1
	DC.W	0x0000					; CSOR1
	DC.W	0x0000					; CSBAR2
	DC.W	0x0000					; CSOR2
	DC.W	0x0000					; CSBAR3
	DC.W	0x0000 					; CSOR3
	DC.W	0x0000					; CSBAR4
	DC.W	0x0000					; CSOR4
	DC.W	0x0000					; CSBAR5
	DC.W	0x0000					; CSOR5
	DC.W	0x0000					; CSBAR6
	DC.W	0x0000					; CSOR6
	DC.W	0x0000					; CSBAR7
	DC.W	0x0000 					; CSOR7
	DC.W	0x0000					; CSBAR8
	DC.W	0x0000					; CSOR8
	DC.W	0x0000					; CSBAR9
	DC.W	0x0000					; CSOR9
	DC.W	0x0000					; CSBAR10
	DC.W	0x0000					; CSOR10

; Common chip select table, refer comment above.

	DC.W	0x0000					; CSBARBT
	DC.W	0x0000					; CSORBT

; New chip select table, refer to comment above.

	DC.W	0x0000					; CSBAR0
	DC.W	0x0000					; CSOR0
	DC.W	0x0000					; CSBAR1
	DC.W	0x0000					; CSOR1
	DC.W	0x0000					; CSBAR2
	DC.W	0x0000					; CSOR2
	DC.W	0x0000					; CSBAR3
	DC.W	0x0000 					; CSOR3
	DC.W	0x0000					; CSBAR4
	DC.W	0x0000					; CSOR4
	DC.W	0x0000					; CSBAR5
	DC.W	0x0000					; CSOR5
	DC.W	0x0000					; CSBAR6
	DC.W	0x0000					; CSOR6
	DC.W	0x0000					; CSBAR7
	DC.W	0x0000 					; CSOR7
	DC.W	0x0000					; CSBAR8
	DC.W	0x0000					; CSOR8
	DC.W	0x0000					; CSBAR9
	DC.W	0x0000					; CSOR9
	DC.W	0x0000					; CSBAR10
	DC.W	0x0000					; CSOR10


; Both the MCR and SYPCR are initialized in the POWER_ON_RESET routine. So the
; following constants will not effect the settings done in POWER_ON_RESET.
;
	DC.W	0x0000					; MCR_OR OR'ed with MCR at power on/reset
	DC.W	0xffff					; MCR_AND AND'ed with MCR at power on/reset

	DC.B	0x00					; SYPCR_OR OR'ed with SYPCR at power on/reset
	DC.B	0xff					; SYPCR_AND AND'ed with SYPCR at power on reset

; CPU32BUG was designed to run on a 68332/68331 using a 32.768 kHz crystal. The SYNCR
; register was then programmed to multiply 32.768 kHz by 512, resulting in a bus frequency
; of 16.777 mHz. CPU32BUG calculates the baud rate divisor for the SCI port assuming the 
; external reference clock will be multiplied by 512. The CANdo 68376 board uses an 4 mHz
; oscillator for the reference, and programs SYNCR to multiply by 4 for a bus frequency 
; of 16 MHz. So, CPU32BUG has to be told a crystal frequency, that when multiplied by 512 
; equals 16 MHz: X * 512 = 16 MHz, solving for X: X = 31.250 KHz, so the following constant
; is set to 31250.
;
	DC.W	31250					; Crystal frequency in Hz SCI baud rate divisors use this
	DC.L	0xffffffff				; ext. clock in Hz. only if MODCLK held low @ reset

	DC.L	0xffffffff				; RB_SP rom auto boot stack pointer value
	DC.L	0xffffffff				; RB_PC rom auto boot program counter value

; default values for SCI set up

	DC.L	0x00001C0F				; PARAMS
	DC.W	0x2580					; BAUD
	DC.B	0x00					; PARITY
	DC.B	0x08					; DATA
	DC.B	0x01					; STOP
	DC.B	0xFF					; XON_ENB
	DC.B	0x11					; XON
	DC.B	0x13					; XOFF

	DC.W	0x0642					; PICR
	DC.W	0x0102					; PITR

; Power on branch vectors
;
; The following 4 branch instructions were executed as part
; of CPU32Bug initialization. None of these branches
; are executed for this incarnation of CPU32Bug.
;
; All initialization is performed in the 
; POWER_ON_RESETroutine.
;
; The following branch is ignored  
	DC.L	0x60ff0000				; 0x100090
	DC.W	0xe056					; PWR_TBL1

; The following branch is ignored  
	DC.L	0x60FF0000				; 0x100096
	DC.W	0xdee8					; PWR_INI

; The following branch is ignored  
	DC.L	0x60ff0000				; 0x10009c
	DC.W	0xe070					; PWR_TBL2

; The following branch is ignored  
	DC.L	0x60FF0000				; 0x1000a2
	DC.W	0x0004					; PWR_TTL

; The last instruction executed in the POWER_ON_RESET code
; is a jump to the following branch instruction. PWR_TST is part of 
; CPU32BUG. PWR_TST will then branch to the PWR_GO branch instruction.
;
PWR_TST_OFFSET:
	DC.L	0x60FF0000				; 0x1000a8
	DC.W	0xd8aa					; PWR_TST

	DC.L	0x60FF0000				; 0x1000ae
	DC.W	0xd4b4					; PWR_GO

	DC.L	0xffffffff				; BRA.L <reserved>
	DC.W	0xffff

	DC.L	0xffffffff				; BRA.L <reserved>
	DC.W	0xffff

	DCB.B	16,0xff					; reserved 16 bytes

; None of the initialization tables are used, since all of the
; setup is done in POWER_ON_RESET. 

; Initialization tables
;
start_tables:
end_tables:
;
	DCB.B	160-(end_tables-start_tables),0xff		; place holder for initialization tables
;
;
; The greeting banner is stored here. 143 bytes are allocated for this banner.
;
	DC.B	banner3-banner1			; character count for sign on message
banner1:
	DC.B	0x0d,0x0a,0x0a			; <CR>,<LF>,<LF>
	DC.B	'Motorola CPU32Bug, adapted to Spectrum Engineering CANdo 68376 board',0x0d,0x0a
	DC.B	'V1.0    www.spectrumengineering.com',0x0d,0x0a,0x0a
banner2:
	DCB.B	143-(banner2-banner1),0x20
banner3:
;
;
; ********************************************************************************************************************
; This ends the CPU32BUG parameter area. The parameter area is EXACTLY 0x200 bytes long.
; The position independent code for CPU32BUG starts at 0x200 and ends at 0xe1cf. So, custom
; code can be placed starting at 0xe1d0.
; ********************************************************************************************************************
;
;
; ********************************************************************************************************************
; Power on reset code for the CANdo 68376 board. Out of reset the 68376 will execute the 
; POWER_ON_RESET code. Part of the initialization code will remap both the SRAM and FLASH memory.
; Then jump to the CPU32BUG code, bypassing the original CPU32BUG initialization code.
; ********************************************************************************************************************
;
	.ORG	0xe1d0		; CPU32BUG code ends at 0xe1cf

POWER_ON_RESET:

; Reset conditions:
; U12 (74HC244A) asserts the following data bus signals during reset:
;
; D0 : logic 1 --> !CSBOOT is configured as a 16 bit port
; D1 : logic 1 --> shared pins !CS0/!BR, !CS1/!BG, !CS2/!BGACK configured as !CS0, !CS1, !CS2
; D2 : logic 1 --> shared pins !CS3/FC0, !CS4/FC1, !CS5/FC2 configured as !CS3, !CS4, !CS5
; D3 : logic 0 --> shared pin  !CS6/ADDR19 configured as ADDR19
; D8 : logic 0 --> PORTE are I/O pins
; D9 : logic 0 --> PORTF are I/O pins
; D11: logic 1 --> slave mode disabled

; D4, D5, D6, D7 : default to logic 1 --> shared high order address pins and chip selects default to chip selects

; System clock:

; Refer to 5.3.1 of the MC68336/376 users manual MC68336/376UM/AD:

; "When MODCLK is held high during reset, the clock synthesizer generates a clock signal from from an external
; reference frequency. The clock synthesizer control register (SYNCR) determines operating frequency and mode of
; operation."

; MODCLK is pulled high using RNBUS1 pin 13. CANdo 68376 does NOT use the on chip crystal oscillator with 
; external crystal. CANdo 68376 uses an external 4MHz oscillator connected to the EXTAL pin of the 68376.

; CANdo 68376 implements the high stability loop filter on pins VDDSYN & XFC.

; So, f(ref) = 4Mhz, and f(sys) is determined by programming the SYNCR register. Details on programming SYNCR
; is in section 5.3.2 of the users manual. Additional SYNCR information is found on page D-8 of users manual.


; the reset state of the SIMCR MM bit is 1, so internal modules are addressed from 0xFFF000..0xFFFFFF


	move.b	#0,simSYPCR		; disable watchdog timer


	move.w	#0x40cf,simMCR	; for SDS debugger and BDM FRZBM = 1


; set the X bit in SYNCR, this controls a divide by two prescaler. Setting X doubles the clock
; speed without changing the VCO speed, so no VCO relock delay is required.
;
; in the SYNCR the following values are:
;
; W = 0 (reset condition)
; X = 1 (by the following instruction)
; Y = 63 (reset condition)
;
; use the formula:  f(sys) = ( f(ref)/128 ) ( 4*(Y+1) * (2^(2*W+x)) )
;
; with f(ref) = 4MHz, then f(sys) = 16MHz
;
; simSYNCR = 0 1 111111 0 0 0 0 0 0 0 0 = 0x7f00

	move.w	#0x7f00,simSYNCR
;
;  THE SYSTEM CLOCK IS NOW 16MHz
;
; ********************************************************************************************************************
; CANdo 68376 memory map information
; ********************************************************************************************************************
;
; The CANdo board can work with 1Mbit OR 4Mbit FLASH memories.
; The CANdo board can work with 1Mbit OR 4Mbit SRAM memories.
;
; 1Mbit = 128Kbytes X 8, 4Mbit = 512Kbytes X 8
;
; SIM programming note:
; For 1Mbit devices the block size is 256K (101),the value of the base address register must be
; an integer multiple of the block size: 0x0, 0x040000, 0x080000, 0x0c0000, 0x100000, ...
;
; For 4Mbit devices the block size is 1M (111), the value of the base address register must be
; an integer multiple of the block size: 0x0, 0x100000, 0x200000, ....
;
; !CSBOOT is the chip select for both FLASH memory devices
; FLASH (2 each 1Mbit devices):     0x100000 ... 0x13ffff  -->  256Kbytes WORD addressable only.
; FLASH (2 each 4Mbit devices):     0x100000 ... 0x1fffff  --> 1024Kbytes WORD addressable only.
;
; !CS0 selects the lower byte (D7 .. D0) of SRAM.
; !CS1 selects the upper byte (D15 .. D8) of SRAM.
;
; SRAM (2 each 1Mbit devices):      0x000000 ... 0x03ffff  -->  256Kbytes WORD & BYTE addressable.
; SRAM (2 each 4Mbit devices):      0x000000 ... 0x0fffff  --> 1024Kbytes WORD & BYTE addressable.
;
; FOR SRAM ONLY:
;
; 128K X 8 devices: Remove  R2 & Install R6.
; 512K X 8 devices: Install R2 &  Remove R6.
;
;
; Chip select pin assignment registers
;
    move.w	#0x00bf,simCSPAR0
	move.w	#0x0155,simCSPAR1
;
; Turn on onchip TPU RAM
;
	move.w	#TRAMBAR_INIT,tpuTRAMBAR
	nop
	nop
	nop
	nop
;
; TPU SRAM is now active with base address as defined by TRAMBAR
;
; At this point only the FLASH memory is being used. Since the FLASH memory is connected to !CSBOOT
; FLASH is mapped into the address space starting at location zero. We want to remap the FLASH to 
; start at location 0x100000, as defined in CSBARBT_INIT. We want to map the SRAM starting
; at location zero.
;
; One way to accomplish this remapping without crashing the system is to execute out of
; an address space whose chip selects we are NOT modifying. So, we will execute the instructions
; that modify the chip select registers out of TPU emulation ram. In order to do this we must
; copy executable code into the TPU emulation ram, then jump to the ram.
;
; Now copy code segment from FLASH memory to TPU SRAM memory, then jump to start
; of code segment in TPU SRAM memory. This code segment will set up the chip select registers
; for both FLASH and SRAM.
;
; a1 points to TPU SRAM, a3 is index register
;
	movea.l	#TPURAMBASE,a1		; TPU base address in a1
	movea.l	#0x0,a3				; use a3 for index register
;
copy_loop:
	move.l	(Start_block,PC,a3.L*4),(a1)+
	addq.l	#1,a3
	cmpa.l	#End_block-Start_block,a3
	bne		copy_loop
;
	jmp		TPURAMBASE


;
; The following section of position independent code is stored in FLASH memory
; then copied to on chip TPU RAM memory, then executed.

	.align	4
Start_block:
; !CS0
	move.w	#CSBAR0_INIT,simCSBAR0
	move.w	#CSOR0_INIT,simCSOR0
; !CS1
	move.w	#CSBAR1_INIT,simCSBAR1
	move.w	#CSOR1_INIT,simCSOR1
; !CSBOOT
	move.w	#CSBARBT_INIT,simCSBARBT
	move.w	#CSORBT_INIT,simCSORBT
;
	moveq.l	#0,d7
;
; We now want to jump to code stored in FLASH, which has just been remapped. So, using the CSBARBT_INIT
; we calculate the base address of FLASH then add the offset to the branch instruction, so CPU32BUG
; will start up.
;
	jmp.l	(FLASHBASE+PWR_TST_OFFSET)
;
	.align	4
End_block:
END_CHECKSUM_COUNT:
